/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea, KC0DGE

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/

/*
 * Database structures
 *
 */

#ifndef __xastir_db_h_
#define __xastir_db_h_

#include <string.h>

/* Define Max stations in memory */
#define MAX_STATIONS 5000


/* define max tracking pos */
#define MAX_TRACKS 100

/* define max tnc line size (should be from tnc.h) */
#define MAX_TNC_LINE_SIZE 300

/* Define APRS Types */
enum APRS_Types {APRS_NULL, APRS_MSGCAP, APRS_FIXED,
		 APRS_DOWN, APRS_MOBILE, APRS_DF,
		 APRS_OBJECT, APRS_WX1, APRS_WX2,
		 APRS_WX3, APRS_WX4, APRS_WX5, APRS_WX6, GPS_RMC, GPS_GGA, GPS_GLL,
                 APRS_DOWN_COMPRESSED, APRS_FIXED_COMPRESSED,
                 APRS_MSGCAP_COMPRESSED };


/* Define Record Types */
#define NORMAL_APRS     'N'
#define MOBILE_APRS     'M'
#define DF_APRS         'D'
#define DOWN_APRS       'Q'
#define NORMAL_GPS_RMC  'C'
#define NORMAL_GPS_GGA  'A'
#define NORMAL_GPS_GLL  'L'

/* define RECORD ACTIVES */
#define RECORD_ACTIVE    'A'
#define RECORD_NOTACTIVE 'N'

/* define data from info type */
#define DATA_VIA_LOCAL 'L' 
#define DATA_VIA_TNC   'T'
#define DATA_VIA_NET   'I'
#define DATA_VIA_FILE  'F'


/* define Heard info type */
#define VIA_TNC         'Y'
#define NOT_VIA_TNC     'N'

/* define Message types */
#define MESSAGE_MESSAGE  'M'
#define MESSAGE_BULLETIN 'B'
#define MESSAGE_NWS      'W'

/* Define file info */
#define MAX_CALLSIGN	 20
#define MAX_TIME         20
#define MAX_PATH         50
#define MAX_LONG	 10
#define MAX_LAT		 9
#define MAX_ALTITUDE	 8
#define MAX_SPEED	 9
#define MAX_COURSE	 7
#define MAX_POWERGAIN	 8
#define MAX_STATION_TIME 10
#define MAX_SAT          4
#define MAX_COMMENTS     80
#define MAX_DISTANCE     10 
#define MAX_WXSTATION    50
#define MAX_TEMP 100
#define MAX_MESSAGE 190

#define MAX_MESSAGES 20
#define MAX_MESSAGE_LENGTH 100
#define MAX_MESSAGE_ORDER 10

#define MAX_BULLETINS  40

extern char my_callsign[MAX_CALLSIGN+1];
extern char my_lat[MAX_LAT];
extern char my_long[MAX_LONG];

typedef struct
{ 
  char active;
  char data_via;
  char type;
  char heard_via_tnc;
  time_t sec_heard; 
  char packet_time[MAX_TIME];
  char call_sign[MAX_CALLSIGN+1];
  char from_call_sign[MAX_CALLSIGN+1];
  char message_line[MAX_MESSAGE_LENGTH+1];
  char seq[MAX_MESSAGE_ORDER+1];
} Message;

#ifdef MSG_DEBUG
extern void mclear_data(Message *clear);
extern void mcopy_data(Message *to, Message *from);
#else
#define mclear_data(clear) memset((Message *)clear, 0, sizeof(Message))
#define mcopy_data(to, from) memmove((Message *)to, (Message *)from, sizeof(Message))
#endif /* MSG_DEBUG */

typedef struct
{
    char aprs_type;
    char aprs_symbol;
    char special_overlay;
} APRS_Symbol;

typedef struct
{
  char active;
  char call_sign[MAX_CALLSIGN+1];
  char record_type;
  char data_via; /* L local, T tnc, I internet, F file */
  char heard_via_tnc;
  int  heard_via_tnc_port;
  time_t heard_via_tnc_last_time; 
  int  last_heard_via_tnc; 
  int  last_port_heard;
  char pass_thru;
  char moving;
  long long_pos;
  long lat_pos;
  int  trail_color;
  int  num_trail;
  long trail_long_pos[MAX_TRACKS+2];
  long trail_lat_pos[MAX_TRACKS+2];
  time_t sec_heard; 
  unsigned int  num_packets;
  char packet_time[MAX_TIME];
  char message_capable;
  APRS_Symbol aprs_symbol;
  char node_path[MAX_PATH];
  char pos_time[MAX_TIME];
  char long_pos_x[MAX_LONG];
  char long_ew;
  char lat_pos_y[MAX_LAT];
  char lat_ns;
  char altitude_time[MAX_TIME];
  char altitude[MAX_ALTITUDE];
  char alt_unit;
  char speed_time[MAX_TIME];
  char speed[MAX_SPEED];
  char speed_unit;
  char course[MAX_COURSE];
  char power_gain[MAX_POWERGAIN];
  char station_time[MAX_STATION_TIME];
  char station_time_type;
  char sats_visible[MAX_SAT];
  time_t wx_sec_time;
  char wx_data;
  char wx_time[MAX_TIME];
  char wx_course_on;
  char wx_course[4];
  char wx_speed_on;
  char wx_speed[4];
  time_t wx_speed_sec_time;
  char wx_gust_on;
  char wx_gust[4];
  char wx_temp_on;  
  char wx_temp[5];
  char wx_rain_on;
  char wx_rain[10];
  char wx_rain_total_on;
  char wx_rain_total[10];
  char wx_snow_on;
  char wx_snow[6];
  char wx_prec_24_on;
  char wx_prec_24[10];
  char wx_prec_00_on;
  char wx_prec_00[10];
  char wx_hum_on;
  char wx_hum[5];
  char wx_baro_on;
  char wx_baro[10];
  char wx_type;
  char wx_station[MAX_WXSTATION];
  char comments[MAX_COMMENTS];
} DataRow;

#ifdef DATA_DEBUG
extern void clear_data(DataRow *clear);
extern void copy_data(DataRow *to, DataRow *from);
#else
#define clear_data(clear) memset((DataRow *)clear, 0, sizeof(DataRow))
#define copy_data(to, from) memmove((DataRow *)to, (DataRow *)from, sizeof(DataRow))
#endif /* DATA_DEBUG */

void mscan_file(char msg_type, void (*function)(Message *fill));

#endif /* __xastir_db_h_ */
