/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea, KC0DGE

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/


#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <sys/file.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/limits.h>
#include <dirent.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <termios.h>
#include <pwd.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <Xm/XmStrDefs.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <string.h>

#include "xastir.h"
#include "maps.h"
#include "alert.h"

#define DOS_HDR_LINES 8
#define GRID_MORE 513

int mag;
int npoints; /* number of points in a line */

/* MAP pointers */

map_vectors *map_vectors_ptr;
text_label *map_text_label_ptr;
symbol_label *map_symbol_label_ptr;


/* MAP counters */

long vectors_num;
long text_label_num;
long object_label_num;


/** MAP DRAWING ROUTINES **/
void draw_grid(Widget w)
{int place;
 char place_str[10];
 int grid_place;
 long xx,yy,xx1,yy1;

/* Set the line width in the GC */
XSetLineAttributes(XtDisplay(w), gc, 1, LineOnOffDash, CapButt,JoinMiter);
XSetForeground(XtDisplay(w),gc,colors[0x08]); 


if(size<GRID_MORE)
  {grid_place=1;}
  else
  {grid_place=10;}

if(long_lat_grid)
  {for(place=180; place>=0;place-=grid_place)
      {sprintf(place_str,"%03d00.00W",place);
       /*printf("Place %s\n",place_str);*/
       xx1 = xx =((convert_long_string(place_str) - x_long_offset)/size);

       if(xx > 0 && xx < screen_width)
         {yy = (convert_lat_string("9000.00N") - y_lat_offset)/size;
          yy1 = (convert_lat_string("9000.00S") - y_lat_offset)/size;
          if(yy<0)
            {yy=0;}
          if(yy1>screen_height)
            {yy1=screen_height;}
          XDrawLine(XtDisplay(w), pixmap,gc,xx,yy,xx1,yy1);
	 }	
      }

   for(place=grid_place; place<181;place+=grid_place)
      {sprintf(place_str,"%03d00.00E",place);
       /*printf("Place %s\n",place_str);*/
       xx1 = xx =((convert_long_string(place_str) - x_long_offset)/size);

       if(xx > 0 && xx < screen_width)
         {yy = (convert_lat_string("9000.00N") - y_lat_offset)/size;
          yy1 = (convert_lat_string("9000.00S") - y_lat_offset)/size;
          if(yy<0)
            {yy=0;}
          if(yy1>screen_height)
            {yy1=screen_height;}
          XDrawLine(XtDisplay(w), pixmap,gc,xx,yy,xx1,yy1);
	 }
      }

   for(place=90; place>=0;place-=grid_place)
      {sprintf(place_str,"%02d00.00N",place);
       /*printf("Place %s\n",place_str);*/
       yy1 = yy =((convert_lat_string(place_str) - y_lat_offset)/size);

       if(yy > 0 && yy < screen_height)
         {xx = (convert_long_string("18000.00W") - x_long_offset)/size;
          xx1 = (convert_long_string("18000.00E") - x_long_offset)/size;
          if(xx<0)
            {xx=0;}
          if(xx1>screen_width)
            {xx1=screen_width;}
          XDrawLine(XtDisplay(w), pixmap,gc,xx,yy,xx1,yy1);
	 }
      }

   for(place=grid_place; place<91;place+=grid_place)
      {sprintf(place_str,"%02d00.00S",place);
       /*printf("Place %s\n",place_str);*/
       yy1 = yy =((convert_lat_string(place_str) - y_lat_offset)/size);

       if(yy > 0 && yy < screen_height)
         {xx = (convert_long_string("18000.00W") - x_long_offset)/size;
          xx1 = (convert_long_string("18000.00E") - x_long_offset)/size;
          if(xx<0)
            {xx=0;}
          if(xx1>screen_width)
            {xx1=screen_width;}
          XDrawLine(XtDisplay(w), pixmap,gc,xx,yy,xx1,yy1);
	 }
      }
  }
}

void map_plot(Widget w, long max_x, long max_y, long x_long_cord, long y_lat_cord, unsigned char color, long object_behavior)
{
    static int redraw_check;
    static XPoint points[MAX_MAP_POINTS];
    static unsigned char last_color = 0;
    static unsigned char last_behavior = 0, first_behavior = 0;
    long x, y;
    int draw_ok;
    unsigned char line_behavior, fill_color;
    char warning[200];

    /* don't ever go over MAX_MAP_POINTS have a bad map not a crashed program*/
    if (npoints > MAX_MAP_POINTS){
	sprintf(warning,"Warning line point count overflow: map_plot\b\n");
	XtAppWarning(app_context, warning);
      npoints = MAX_MAP_POINTS;
    }

    /* if map_color_levels are on see if we should draw the line? */
    draw_ok=0;
    if (map_color_levels)
       switch(color) {
         case(0x01):
         case(0x14):
         case(0x18):
              if(mag<100)
               draw_ok=1;
              break;
         case(0x15):
         case(0x19):
              if(mag<600)
                draw_ok=1;
              break;
         case(0x16):
              if(mag<800)
                draw_ok=1;
              break;
         default:
              draw_ok=1;
              break;
       } else {draw_ok=1;}

  if (draw_ok){
    x = ((x_long_cord - x_long_offset)/size);
    y = ((y_lat_cord - y_lat_offset)/size);
    if (x < -MAX_OUTBOUND) x = -MAX_OUTBOUND;
    if (y < -MAX_OUTBOUND) y = -MAX_OUTBOUND;
    if (x > max_x) x = max_x;
    if (y > max_y) y = max_y;
    if (debug_level & 16)
	  printf(" MAP Plot - max_x: %ld, max_y: %ld, x: %ld, y: %ld, color: %d, behavior: %lx\n",
	     max_x, max_y, x, y, color, object_behavior);
    if (last_color != color || color == 0xff) {
	if (npoints && last_color != 0xff) {
		line_behavior = last_behavior;
	    if (last_behavior & 0x80) {
		    if (color) {
			fill_color = (last_behavior&~0x80) + 0x60;
			if (fill_color > 0x69) fill_color = 0x60;
		    }
		    else
		      fill_color = object_behavior;
		XSetForeground(XtDisplay(w), gc, colors[fill_color]);
		XFillPolygon(XtDisplay(w), pixmap, gc, points, npoints, Complex, CoordModeOrigin);
		    line_behavior = first_behavior;
	    }
		if (line_behavior & 0x01)
		  XSetLineAttributes(XtDisplay(w), gc, 2, LineSolid, CapButt,JoinMiter);
	    else
		  XSetLineAttributes(XtDisplay(w), gc, 1, LineSolid, CapButt,JoinMiter);
		if (color==0x56)
		  XSetLineAttributes(XtDisplay(w), gc, 10, LineSolid, CapButt,JoinMiter);
		XSetForeground(XtDisplay(w), gc, colors[last_color]);
	    XDrawLines(XtDisplay(w), pixmap, gc, points, npoints, CoordModeOrigin);
	    npoints = 0;
	    /* check to see if we have been away from the screen too long */
	    if(redraw_check > 1000) {
		redraw_check=0;
		XmUpdateDisplay(XtParent(da));
	    }
	    redraw_check++;
      }
      last_color = color;
	    if (color == 0xff) {
		npoints = 0;
		first_behavior = object_behavior;
	    }
	points[npoints].x = x;
	points[npoints].y = y;
	if (points[npoints].x > (-MAX_OUTBOUND) && points[npoints].x < max_x &&
	    points[npoints].y > (-MAX_OUTBOUND) && points[npoints].y < max_y && color != 0) npoints++;
	last_behavior = object_behavior;
      return;
    }
    points[npoints].x = x;
    points[npoints].y = y;
    last_behavior = object_behavior;

    if (points[npoints].x != points[npoints-1].x || points[npoints].y != points[npoints-1].y) {
	if (last_behavior & 0x80) {
	  npoints++;
	}
	else if (points[npoints].x > (-MAX_OUTBOUND) && points[npoints].x < max_x && 
		 points[npoints].y > (-MAX_OUTBOUND) && points[npoints].y < max_y) {	  
	      npoints++;
	}
    }
  }
  else
      npoints = 0;
} /* map_plot */


void draw_map(Widget w, char *dir, char *filenm, int tn, alert_entry *alert, unsigned char alert_color)
{
FILE *f;
char file[2000];
char map_it[300];

/* map header info */
char map_type[5];
char map_version[5];
char file_name[33];
char map_title[33];
char map_creator[8];
unsigned long creation_date;
 unsigned long left_boundary;
 unsigned long right_boundary;
 unsigned long top_boundary;
 unsigned long bottom_boundary;
char map_reserved1[9];
long total_vector_points;
long total_labels;
char map_reserved2[141];
 char Buffer[2049];
char *ptr;
 int dos_labels;
long temp;
int points_per_degree;
int map_range;

/* vector info */

unsigned char vector_start;
unsigned char object_behavior;
 unsigned long x_long_cord;
 unsigned long y_lat_cord;

/* label data */
char label_type[3];
 unsigned long label_x_cord;
 unsigned long label_y_cord;
int  temp_mag;
int  label_mag;
char label_symbol_del;
char label_symbol_char;
char label_text_color;
char label_text[50];

unsigned long year;
unsigned long days;

long count;
int label_length;
int i;
int map_maxed_vectors;
int map_maxed_text_labels;
int map_maxed_symbol_labels;
map_vectors *vectors_ptr;
text_label *text_ptr;
symbol_label *symbol_ptr;

int line_width;

int x, y;

 unsigned long max_x_long_offset;
 unsigned long max_y_lat_offset;

long map_border_min_x;
long map_border_max_x;

long map_border_min_y;
long map_border_max_y;

int color;

long max_x, max_y;

long x_test,y_test;
 int in_window = 0;

x=0;
y=0;
 color=-1;

line_width = 1;
mag = (1 * size)/2;

/* MAP counters */

vectors_ptr = map_vectors_ptr;
text_ptr = map_text_label_ptr;
symbol_ptr = map_symbol_label_ptr;

map_maxed_vectors = 0;
map_maxed_text_labels = 0;
map_maxed_symbol_labels = 0;
 npoints = 0;

sprintf(file,"%s/%s",dir,filenm);

f=fopen(file,"r");
if(f!=NULL)
  {
   fread(map_type,4,1,f);   
   map_type[4]='\0';

   dos_labels = FALSE;
   points_per_degree = 300;
   if (strtod(map_type, &ptr) != 0.0 && (*ptr == '\0' || *ptr == ' ')) { /* DOS type map */
       int j;
       top_boundary = left_boundary = bottom_boundary = right_boundary = 0;
       rewind(f);
       map_title[0] = map_creator[0] = Buffer[0] = '\0';
       strncpy(map_type, "DOS ", 4);
       strncpy(file_name, filenm, 32);
       total_vector_points = 30000;
       total_labels = 2000;
       for (j = 0; j < DOS_HDR_LINES; strlen(Buffer) ? : j++)
       {
	   fgets(&Buffer[strlen(Buffer)], sizeof(Buffer)-(strlen(Buffer)), f);
	   while ((ptr = strpbrk(Buffer, "\r\n")) != NULL && j < DOS_HDR_LINES)
          {
              *ptr = '\0';
              for (ptr++; *ptr == '\r' || *ptr == '\n'; ptr++);
              switch (j)
              {
                case 0:
		   top_boundary = (unsigned long)(-atof(Buffer)*360000+32400000);
                  break;
                case 1:
		   left_boundary = (unsigned long)(-atof(Buffer)*360000+64800000);
                  break;
                case 2:
		   points_per_degree = (long)atof(Buffer);
                  break;
                case 3:
		   bottom_boundary = (unsigned long)(-atof(Buffer)*360000+32400000);
                  bottom_boundary = bottom_boundary + bottom_boundary - top_boundary;
                  break;
                case 4:
		   right_boundary = (unsigned long)(-atof(Buffer)*360000+64800000);
                  right_boundary = right_boundary + right_boundary - left_boundary;
                  break;
                case 5:
		   map_range = (long)atof(Buffer);
                  break;
                case 7:
		   strncpy(map_version, Buffer, 4);
                  break;
              }
	       strcpy(Buffer, ptr);
	       if (strlen(Buffer)) j++;
          }
       }
   } else {  
   fread(map_version,4,1,f);
   map_version[4]=0;

   fread(file_name,32,1,f);
   file_name[32]=0;

   fread(map_title,32,1,f);
   map_title[32]=0;
   if(debug_level & 2)
     {printf("Map Title %s\n",map_title);}
   
   fread(map_creator,8,1,f);
   map_creator[8]=0;
   if(debug_level & 2)
     {printf("Map Creator %s\n",map_creator);}

   fread(&temp,4,1,f);
   creation_date=ntohl(temp);
   if(debug_level & 2)
     {printf("Creation Date %lX\n",creation_date);}

   year=creation_date/31536000l;

   days=(creation_date-(year*31536000l))/86400l;

   if(debug_level & 2)
     {printf("year is %ld + days %ld\n",1904l+year,days);}

   fread(&temp,4,1,f);
       left_boundary=ntohl(temp);

   fread(&temp,4,1,f);
       right_boundary=ntohl(temp);
  
   fread(&temp,4,1,f);
       top_boundary=ntohl(temp);

   fread(&temp,4,1,f);
       bottom_boundary=ntohl(temp);

       if (strcmp(map_version, "2.00") != 0) {
	   left_boundary *= 10;
	   right_boundary *= 10;
	   top_boundary *= 10;
	   bottom_boundary *= 10;
       }
   fread(map_reserved1,8,1,f);

   fread(&temp,4,1,f);
   total_vector_points=ntohl(temp);

   fread(&temp,4,1,f);
   total_labels=ntohl(temp);

   fread(map_reserved2,140,1,f);
   }
     
   if(debug_level & 2) {
       printf("Map Type %s, Version: %s, Filename %s\n", map_type, map_version, file_name);
       printf("Left Boundary %ld, Right Boundary %ld\n", left_boundary, right_boundary);
       printf("Top Boundary %ld, Bottom Boundary %ld\n", top_boundary, bottom_boundary);
       printf("Total vector points %ld, total labels %ld\n", total_vector_points, total_labels);
   }

   if (alert) {
       strncpy(alert->filename, filenm, sizeof(alert->filename));
       strcpy(alert->title, map_title);
       alert->top_boundary = top_boundary;
       alert->bottom_boundary = bottom_boundary;
       alert->left_boundary = left_boundary;
       alert->right_boundary = right_boundary;
   }

   max_x_long_offset=x_long_offset+(screen_width*size); 
   max_y_lat_offset=y_lat_offset+(screen_height*size);

   if (debug_level & 4) {
       printf("x_long_offset: %ld, y_lat_offset: %ld, max_x_long_offset: %ld, max_y_lat_offset: %ld, tn: %d\n",
	      x_long_offset, y_lat_offset, max_x_long_offset, max_y_lat_offset, tn);
   }

   
   if(!tn)
     {/* check if auto map */
      in_window=1;
     }
     else
     {/* check edges and corners - quick check*/
      if( ((left_boundary<=x_long_offset) && (x_long_offset<=right_boundary) && (top_boundary<=y_lat_offset) && (y_lat_offset<=bottom_boundary)) ||
          ((left_boundary<=x_long_offset) && (x_long_offset<=right_boundary) && (top_boundary<=max_y_lat_offset) && (max_y_lat_offset<=bottom_boundary)) ||
          ((left_boundary<=max_x_long_offset) && (max_x_long_offset<=right_boundary) && (top_boundary<=y_lat_offset) && (y_lat_offset<=bottom_boundary)) ||
          ((left_boundary<=max_x_long_offset) && (max_x_long_offset<=right_boundary) && (top_boundary<=max_y_lat_offset) && (max_y_lat_offset<=bottom_boundary)) ||
          ((x_long_offset<=left_boundary) && (left_boundary<=max_x_long_offset) && (y_lat_offset<=top_boundary) && (top_boundary<=max_y_lat_offset)) ||
          ((x_long_offset<=left_boundary) && (left_boundary<=max_x_long_offset) && (y_lat_offset<=bottom_boundary) && (bottom_boundary<=max_y_lat_offset)) ||
          ((x_long_offset<=right_boundary) && (right_boundary<=max_x_long_offset) && (y_lat_offset<=top_boundary) && (top_boundary<=max_y_lat_offset)) ||
          ((x_long_offset<=right_boundary) && (right_boundary<=max_x_long_offset) && (y_lat_offset<=bottom_boundary) && (bottom_boundary<=max_y_lat_offset))  
	)
        {in_window=1;}
        else
        {/* find min and max borders to look at */
         /* this routine are for those odd sized maps */
         if(left_boundary>x_long_offset)
           {map_border_min_x=left_boundary;}
           else
           {map_border_min_x=x_long_offset;}

         if(right_boundary<max_x_long_offset)
           {map_border_max_x=right_boundary;}
           else
           {map_border_max_x=max_x_long_offset;}

         if(top_boundary>y_lat_offset)
           {map_border_min_y=top_boundary;}
           else
           {map_border_min_y=y_lat_offset;}

         if(bottom_boundary<max_y_lat_offset)
           {map_border_max_y=bottom_boundary;}
           else
           {map_border_max_y=max_y_lat_offset;}
    
         /* do difficult check in side map */
         for(x_test=map_border_min_x; (x_test<=map_border_max_x && !in_window); x_test+=((size*screen_width)/10))
            {for(y_test=map_border_min_y; (y_test<=map_border_max_y && !in_window); y_test+=((size*screen_height)/10))
                {if((x_long_offset<=x_test) && (x_test<=max_x_long_offset) &&
                    (y_lat_offset<=y_test) && (y_test<=max_y_lat_offset)
                   )
                   {in_window=1;}
		}
	    }
	}
     }
      
   if (alert) alert->flags[0] = in_window ? 'Y': 'N';

   if(in_window && !alert)
     {
	 unsigned char last_behavior, special_fill = False;
	 object_behavior = 0;
      sprintf(map_it,langcode("BBARSTA028"),filenm);     
      if(debug_level & 1) {
	  printf("in Boundary %s\n", map_it);
	 }

     XmTextFieldSetString(text,map_it);

     XSetLineAttributes(XtDisplay(w), gc, line_width, LineSolid, CapButt,JoinMiter);

     /* read vectors */
     max_x = screen_width+MAX_OUTBOUND; 
     max_y = screen_height+MAX_OUTBOUND;

   x_long_cord = y_lat_cord = color = 0;
   for(count=0l; count<total_vector_points && !feof(f) && !dos_labels; count++)
        {
         if (strncmp("DOS ", map_type, 4) == 0)
            {
	   fgets(&Buffer[strlen(Buffer)], sizeof(Buffer)-(strlen(Buffer)), f);
	   while ((ptr = strpbrk(Buffer, "\r\n")) != NULL && !dos_labels)
	   {
	       long LatHld = 0, LongHld;
	       char *trailer;
	       *ptr = '\0';
	       for (ptr++; *ptr == '\r' || *ptr == '\n'; ptr++);
	       
	   process:
	       if (strncasecmp("Line", map_version, 4) == 0)
	       {
		   int k;
		   color = strtol(Buffer, &trailer, 0);
		   if (trailer != NULL && *trailer == ',')
		   {
		       trailer++;
		       if (color == -1)
		       {
			   dos_labels = TRUE;
			   strcpy(Buffer, ptr);
			   break;
		       }
		       for (k = strlen(trailer)-1; k >= 0; k--) trailer[k] = (unsigned char)trailer[k] - 27;
		       object_behavior = *(unsigned char *)trailer;
		       while (*trailer)
		       {
			   LongHld = (*(unsigned char *)trailer)*16;
			   trailer++;
			   LatHld = (*(unsigned char *)trailer)*8;
			   trailer++;
			   LongHld += (*trailer>>3)&0xf;
			   LatHld += (*trailer)&7;
			   trailer++;
			   LatHld = LatHld*(360000/points_per_degree);
			   LongHld = LongHld*(360000/points_per_degree);
			   x_long_cord = LongHld + left_boundary;
			   y_lat_cord = LatHld + top_boundary;
			   map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, color, 0);
		       }
		       map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, 0, 0);
		   }
	       }
	       else if (strncasecmp("ASCII", map_version, 4) == 0)
	       {
		   if (color == 0) 
		   {
		       color = strtol(Buffer, &trailer, 0);
		       if (trailer && strpbrk(trailer, ", ") != NULL)
		       {
			   for (; *trailer == ',' || *trailer == ' '; trailer++);
			   object_behavior = strtol(trailer, &trailer, 0);
			   if (object_behavior == 0xff) dos_labels = TRUE;
		       }
		   }
		   else
		   {
		       LongHld = strtol(Buffer, &trailer, 0);
		       if (trailer && strpbrk(trailer, ", ") != NULL)
		       {
			   for (; *trailer == ',' || *trailer == ' '; trailer++);
			   LatHld = strtol(trailer, &trailer, 0);
		       }
		       else if (LongHld == 0 && *trailer != '\0')
			   {
			    strncpy(map_version, "Comp", 4);
			       goto process;
			   }
		       if (LongHld == 0 && LatHld == 0) {
			 color = 0;
			 map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, color, 0);
		       }
		       else if (LongHld == 0 && LatHld == -1) {
			 dos_labels = TRUE;
			 map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, 0, 0);
		       }
		       else
		       {
			   LatHld = LatHld*(360000/points_per_degree);
			   LongHld = LongHld*(360000/points_per_degree);
			   x_long_cord = LongHld + left_boundary;
			   y_lat_cord = LatHld + top_boundary;
			   map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, color, 0);
		       }
		   }
	       }
	       else if (strncasecmp("Comp", map_version, 4) == 0)
	       {
		   char Tag[81];
		   int k;
		   Tag[80] = '\0';
		   if (color == 0) 
		   {
		       color = strtol(Buffer, &trailer, 0);
		       if (trailer != NULL && strpbrk(trailer, ", ") != NULL)
		       {
			   for (; *trailer == ',' || *trailer == ' '; trailer++);
			   object_behavior = strtol(trailer, &trailer, 0);
			   strncpy(Tag, trailer, 80);
			   if (object_behavior == 0xff) dos_labels = True;
		       }
		   }
		   else
		   {
		       LongHld = strtol(Buffer, &trailer, 0);
		       for (; *trailer == ',' || *trailer == ' '; trailer++);
		       LatHld = strtol(trailer, &trailer, 0);
		       if (LatHld == 0 && *trailer != '\0')
			 LatHld = 1;
		       if (LongHld == 0 && LatHld == 0)
		       {
			   color = 0;
			   map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, color, 0);
		       }
		       else if (LongHld == 0 && LatHld == -1)
		       {
			   dos_labels = True;
			   map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, color, 0);
		       }
		       if (color && !dos_labels)
		       {
			   trailer = Buffer;
		   for (k = strlen(trailer)-1; k >= 0; k--) trailer[k] = (unsigned char)trailer[k] - 27;
		   object_behavior = *(unsigned char *)trailer;
		   while (*trailer)
		   {
		       LongHld = (*(unsigned char *)trailer)*16;
		       trailer++;
		       LatHld = (*(unsigned char *)trailer)*8;
		       trailer++;
		       LongHld += (*trailer>>3)&0xf;
		       LatHld += (*trailer)&7;
		       trailer++;
		       LatHld = LatHld*(360000/points_per_degree);
		       LongHld = LongHld*(360000/points_per_degree);
		       x_long_cord = LongHld + left_boundary;
		       y_lat_cord = LatHld + top_boundary;
			       map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, color, 0);
			   }
		       }
		   }
	       }
	       else
	       {
		   LongHld = strtol(Buffer, &trailer, 0);
		   if (trailer)
		   {
		       if (*trailer == ',' || *trailer ==  ' ')
		       {
			   if (LongHld == 0) strncpy(map_version, "ASCII", 4);
			   trailer++;
			   object_behavior = strtol(trailer, &trailer, 0);
			   if (object_behavior == 0xff) dos_labels = TRUE;
			   if (object_behavior == 0 && *trailer != '\0')
			   {
			       strncpy(map_version, "Line", 4);
			       goto process;
			   }
			   color = LongHld;
		       }
		   }
		   else
		     strncpy(map_version, "Comp", 4);
	       }
	       strcpy(Buffer, ptr);
	   }
            }
            else
            {
	   last_behavior = object_behavior;
             fread(&vector_start,1,1,f);
             fread(&object_behavior,1,1,f);  

             if (strcmp(map_type, "COMP") == 0)
	        {
	         short temp_short;
	         long LatOffset, LongOffset;
	       LatOffset = top_boundary - top_boundary%6000;
	       LongOffset = left_boundary - left_boundary%6000;
	         fread(&temp_short, 2, 1, f);
	       x_long_cord=(ntohs(temp_short) * 10 + LongOffset);
	         fread(&temp_short, 2, 1, f);
	       y_lat_cord=(ntohs(temp_short) * 10 + LatOffset);
		}
                else
	        {
                 fread(&temp,4,1,f);
	       x_long_cord = ntohl(temp);
	       if (strcmp(map_version, "2.00") != 0)
		 x_long_cord *= 10;

                 fread(&temp,4,1,f);
	       y_lat_cord = ntohl(temp);
	       if (strcmp(map_version, "2.00") != 0)
		 y_lat_cord *= 10;
		}

	   if (alert_color && last_behavior & 0x80 && vector_start == 0xff) {
	       map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, 0, alert_color);
	       //special_fill = True;
	   }
	  map_plot(w, max_x, max_y, x_long_cord, y_lat_cord, vector_start, object_behavior);
       }
	}

   if (alert_color)
     map_plot(w, max_x, max_y, 0, 0, 0, special_fill ? 0xfd: alert_color);
   else
     map_plot(w, max_x, max_y, 0, 0, 0xff, 0);
      XSetForeground(XtDisplay(w),gc,colors[20]);
      line_width=2;
      XSetLineAttributes(XtDisplay(w), gc, line_width, LineSolid, CapButt,JoinMiter); 
      /* read labels */
   for(count=0l; count<total_labels && !feof(f);count++) {
	if (strcmp(map_type, "DOS ") == 0) {
	    char * trailer;
	    fgets(&Buffer[strlen(Buffer)], sizeof(Buffer)-(strlen(Buffer)), f);
	    for (;(ptr = strpbrk(Buffer, "\r\n")) != NULL; strcpy(Buffer, ptr))
	    {
		*ptr = '\0';
		label_type[0] = 0x08;
		for (ptr++; *ptr == '\r' || *ptr == '\n'; ptr++);
		trailer = strchr(Buffer, ',');
		if (trailer && strncmp(Buffer, "0", 1) != 0)
		{
		    *trailer = '\0';
		    trailer++;
		    strcpy(label_text, Buffer);
		    label_length=strlen(label_text);
		    label_y_cord = (long)(-strtod(trailer, &trailer)*360000) + 32400000;
		    trailer++;
		    label_x_cord = (long)(-strtod(trailer, &trailer)*360000) + 64800000;
		    trailer++;
		    label_mag = strtol(trailer, &trailer, 0) * 20;
		    if((label_type[0] & 0x80)==0)
		    {/* left of coords */
			x = ((label_x_cord - x_long_offset)/size)-(label_length*6);
		    }
		    else
		    {/* right of coords */
			x = ((label_x_cord - x_long_offset)/size);
		    }
      
		    y = ((label_y_cord - y_lat_offset)/size); 
         
		    if(x > (0) && (x< (screen_width)))
		    {if(y > (0) && (y< (screen_height)))
		    {/*printf("Label mag %d mag %d\n",label_mag,(size*2)-1);*/
			if(label_mag>((size*2)-1) || label_mag==0)  
			{XSetForeground(XtDisplay(w),gc,colors[0x0ff]); 
			 XFillRectangle(XtDisplay(w),pixmap,gc,x-1,(y-10),(label_length*6)+2,11);

			 XSetForeground(XtDisplay(w),gc,colors[(label_type[0] & 0x7f)]); 
			 XDrawString(XtDisplay(w), pixmap,  gc, x, y, label_text, label_length);
			}
		    }
		    }      
		}
	    }
	}
	else {
	 fread(label_type,1,2,f);
          if(label_type[1]==0)
            {/* label is text */
             fread(&temp,4,1,f);
          label_x_cord=ntohl(temp);
	  if (strcmp(map_version, "2.00") != 0)
	    label_x_cord *= 10;
             fread(&temp,4,1,f);
          label_y_cord=ntohl(temp);
	  if (strcmp(map_version, "2.00") != 0)
	    label_y_cord *= 10;
             fread(&temp_mag,2,1,f);
          label_mag=ntohs(temp_mag);
	  if (strcmp(map_version, "2.00") != 0)
	    label_mag *= 10;
	     if (strcmp(map_type, "COMP") == 0)
	     for (i = 0; i < 32; i++)
	        {
	         fread(&label_text[i], 1, 1, f);
	         if (label_text[i] == '\0')
	 	    break;
		}
	        else
                fread(label_text,32,1,f);
             label_length=strlen(label_text);

             for(i=(label_length-1); i>0 ;i--)
                {if(label_text[i]==' ')
                   {label_text[i]='\0';}
                   else
                   {break;}            
		}
	 
             label_length=strlen(label_text);
             /*printf("labelin:%s\n",label_text);*/

	 
             if((label_type[0] & 0x80)==0)
               {/* left of coords */
                x = ((label_x_cord - x_long_offset)/size)-(label_length*6);
                x=0; 
	       }
               else
               {/* right of coords */
                x = ((label_x_cord - x_long_offset)/size);
	       }
      
             y = ((label_y_cord - y_lat_offset)/size); 
         
             if(x > (0) && (x< (screen_width)))
               {if(y > (0) && (y< (screen_height)))
                  {/*printf("Label mag %d mag %d\n",label_mag,(size*2)-1);*/
                   if(label_mag>((size*2)-1) || label_mag==0)  
                     {XSetForeground(XtDisplay(w),gc,colors[0x0ff]); 
                      XFillRectangle(XtDisplay(w),pixmap,gc,x-1,(y-10),(label_length*6)+2,11);

                      XSetForeground(XtDisplay(w),gc,colors[(label_type[0] & 0x7f)]); 
                      XDrawString(XtDisplay(w), pixmap,  gc, x, y, label_text, label_length);
		     }
		  }
	       }
	    }
            else
            {/* label is symbol */
             fread(&temp,4,1,f);
          label_x_cord=ntohl(temp);
	  if (strcmp(map_version, "2.00") != 0)
	    label_x_cord *= 10;
             fread(&temp,4,1,f);
          label_y_cord=ntohl(temp);
	  if (strcmp(map_version, "2.00") != 0)
	    label_y_cord *= 10;
             fread(&temp_mag,2,1,f);
          label_mag=ntohs(temp_mag);  
	  if (strcmp(map_version, "2.00") != 0)
	    label_mag *= 10;
             fread(&label_symbol_del,1,1,f);
             fread(&label_symbol_char,1,1,f);
             fread(&label_text_color,1,1,f);
             if (strcmp(map_type, "COMP") == 0)
               for (i = 0; i < 32; i++)
                {
                 fread(&label_text[i], 1, 1, f);
                 if (label_text[i] == '\0')
                    break;
		}
              else 
                fread(label_text,29,1,f);
             /* Do object */
                   
	    }
      
	 }
     }
  }
  }
if(f!=NULL) 
  {fclose(f);}
XmUpdateDisplay(XtParent(da));
}

void  map_search(Widget w, char *dir, alert_entry *alert, int *alert_count)
{struct direct *dl;
 DIR *dm;
 char fullpath[8000];
 struct stat nfile;
 const time_t *ftime; 
 char this_time[40];
 char *ptr;
 int line_width;
 char *map_dir;
 int map_dir_length;

line_width=1;

/* Set the line width in the GC */
XSetLineAttributes(XtDisplay(w), gc, line_width, LineSolid, CapButt, JoinMiter);

 map_dir = alert ? ALERT_MAP_DIR: WIN_MAP_DIR;
 map_dir_length = strlen(map_dir);

 dm = opendir(dir);
 if(!dm) {
     sprintf(fullpath, "aprsmap %s", dir);
     perror(fullpath);
 }
 else {
     int count = 0;
     while((dl = (struct direct *)readdir(dm))) {
	 sprintf(fullpath,"%s/%s",dir,dl->d_name);
          /*printf("FULL PATH %s\n",fullpath);*/
	 if(stat(fullpath,&nfile)==0) {
	     ftime=&nfile.st_ctime;
	     switch(nfile.st_mode & S_IFMT) {
	       case(S_IFDIR):/*printf("file %c letter %c\n",dl->d_name[0],letter);*/
		 if((strcmp(dl->d_name,".") !=0) && (strcmp(dl->d_name,"..") !=0)) {
		     strcpy(this_time,ctime(ftime));
		     map_search(w, fullpath, alert, alert_count);
                                    }
                                   break;
                    case(S_IFREG):/*printf("FILE %s\n",dl->d_name);*/
		 if (strncmp(fullpath, map_dir, map_dir_length) != 0) {
		     draw_map(w,dir,dl->d_name,1, alert ? &alert[*alert_count]: NULL, 0);
		     if (alert_count && *alert_count) (*alert_count)--;
		 }
		 else {
		     for (ptr = &fullpath[map_dir_length]; *ptr == '/'; ptr++);
		     draw_map(w, map_dir, ptr, 1, alert ? &alert[*alert_count]: NULL, 0);
		     if (alert_count && *alert_count) (*alert_count)--;
		 }
		 count++;
                                  break;
                    default:break;
		   }
	    }
	 }
     if (debug_level & 8) printf("Number of maps queried: %d\n", count);
     closedir(dm);
   }

}

void load_alert_maps(Widget w, char *dir)
{
#define MAX_ALERT 7000
    int i, level;
    alert_entry alert[MAX_ALERT];
    int alert_count = MAX_ALERT-1;
    char alert_scan[400], *dir_ptr;

    /* gray86, red2, yellow2, cyan2, RoyalBlue, ForestGreen, orange3 */
    unsigned char fill_color[] = {0x69, 0x4a, 0x63, 0x66, 0x61, 0x64, 0x62};

    if (alert_message_scan()) {
	memset(alert_scan, 0, sizeof(alert_scan));
	strcpy(alert_scan, dir);
	strcat(alert_scan, "/");
	dir_ptr = &alert_scan[strlen(alert_scan)];
	for (i = 0; i < strlen(alert_tag); i += 2) {
	    *dir_ptr = alert_tag[i];
	    *(dir_ptr+1) = alert_tag[i+1];
	    map_search(w, alert_scan, alert, &alert_count);
	}
    }
    if (!alert_count) XtAppWarning(app_context, "Alert Map count overflow: load_alert_maps\b\n");
    for (i = MAX_ALERT-1; i>alert_count; i--)
	if (alert[i].flags[0] == 'Y' || alert[i].flags[0] == 'N')
	  alert_update_list(&alert[i], 0);
    for (i = 0; i < alert_list_count; i++)
      if (alert_list[i].filename[0]) {
	  alert[0] = alert_list[i];
	  draw_map(w, dir, alert_list[i].filename, 1, &alert[0], 0);
	  alert_update_list(&alert[0], 2);
      }
    alert_sort_active();
    for (i = alert_list_count -1; i >= 0; i--)
      if (alert_list[i].flags[0] == 'Y' && (level = alert_active(&alert_list[i], 2))) {
	    if (level >= sizeof(fill_color))
	      level = 0;
	  draw_map(w, dir, alert_list[i].filename, 0, NULL, fill_color[level]);
      }
    alert_display_request();
}

void load_auto_maps(Widget w, char *dir)
{
map_search(w, dir, NULL, NULL);
draw_grid(w);
}

void load_maps(Widget w)
{
FILE *f;
char mapname[300];

if(debug_level & 1)
   {printf("Load maps start\n");}

f=fopen(WIN_MAP_DATA,"r");
if(f!=NULL)
  {if(debug_level & 1)
   {printf("Load maps Open map file\n");}
   while(!feof(f))
        {if(fscanf(f,"%s",mapname)==1)
           {if(mapname[0]!='#')
              {draw_map(w,WIN_MAP_DIR,mapname, 0, NULL, 0);
               if(debug_level & 1)
                 {printf("Load maps -%s\n",mapname);}
               XmUpdateDisplay(da);
              }
	   }
	}
  }
  else
  {printf("Could not open map file %s\n",WIN_MAP_DATA);}

if(f!=NULL) 
  {fclose(f);}

draw_grid(w);

if(debug_level & 1)
   {printf("Load maps stop\n");}
}
