/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea, KC0DGE

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/

#include <string.h>
#include <stdlib.h>
#include <termios.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>

#include "main.h"
#include "xastir.h"

char wx_station_type[100];

char raw_wx_string[300];

extern void fill_wx_data(void);

extern char *get_time(char *time_here);
extern time_t sec_now(void);
extern int redraw_on_new_data;

/* from interface.c */
extern void output_my_data(char *message, int port, int type);

/* from db.c */
void mem_replace_data(DataRow fill, int record_num);

/* from lang.c */
extern char *langcode(char *code);
extern char langcode_hotkey(char *code);

#define MAX_WX_STRING 300
#define WX_TYPE 'z'

float rain[24];
float rain_00;
float rain_24;
float rain_base[24];
int rain_first[24];



/***********************************************************/
/* return the hour of time                                 */
/***********************************************************/
int hour(void)
{struct tm *time_now;
 time_t secs_now;
 char shour[5];

 secs_now=sec_now();
 time_now = localtime(&secs_now);
 strftime(shour,4,"%H",time_now);
 
 return(atoi(shour));
}

/***********************************************************/
/* clear rain data                                         */
/***********************************************************/
void clear_rain_data(void)
{int x;
 for (x=0;x<24;x++) {
     rain[x]=0.0; 
     rain_base[x]=0.0;
     rain_first[x]=1; 
 }
rain_00=0.0;
rain_24=0.0;

}

/***********************************************************/
/* compute_rain - compute rain total from the total rain so*/
/* far                                                     */
/***********************************************************/
    
void compute_rain(float rain_total)
{int h;
 int x;
 int i;
 float r;
 float r_first;

 
 r_first=0.0;
 h=hour();

 if (rain_first[h]) {
     rain_first[h]=0;
     rain_base[h]=rain_total;
     i=h+1;
     if (h>23)
        i=0;
     rain_first[i]=1;
 }
 
 /* set total rain for this hour */
 rain[h]=rain_total-rain_base[h];

 /* set total rain in last 24 hours */
 r=0.0;
 i=h+1;
 if (i>23)
    i=0;
 for (x=0;x<=24;x++) {       
     r+=rain[i++];
     if (i>23)
        i=0;
 }
 rain_24=r;

 /* set rain since midnight */
 r=0.0;
 for (x=1;x<=h;x++) {
     r+=rain[x];
 }
 rain_00=r;

}


/***********************************************************/
/* compute_gust - compute wind gusts                       */
/*                                                         */
/***********************************************************/
float compute_gust(float wx_speed, float last_speed, time_t *last_speed_time)
{float computed_gust;

 /* make sure speed time is not zero */
 if(*last_speed_time==0)
   *last_speed_time=sec_now(); 

 /* set computed to last gust speed */
 computed_gust=last_speed;

 /* check to see if gust is with in 5 min*/
 if ((*last_speed_time+300) > sec_now()) {
    /* speed time within 5 min */

    /* see if gust is higher than last */ 
    if (wx_speed >= last_speed) 
       computed_gust=wx_speed;
     		
 } else { 
   /* speed time over 5 min */

   /* since is was over 5 min reset gust */
   computed_gust=wx_speed;
   *last_speed_time=sec_now(); 
 }

return(computed_gust);
}

/***********************************************************/
/* wx fill data field                                      */
/* from: 0=local station, 1=regular decode (other stations)*/
/* type: type of WX packet                                 */
/* data: the packet of WX data                             */
/* fill: the station data to fill                          */
/***********************************************************/
void wx_fill_data(int from, int type, char *data, DataRow *fill)
{time_t last_speed_time;
 float last_speed;
 float computed_gust;
 int  temp1;
 int  temp2;
 int  temp3;
 char temp[20];
 char temp_data1[10];
 char *temp_conv;
 int len;
 
 last_speed=0.0;
 computed_gust=0.0;
 last_speed_time=0;
 len=strlen(data);

 switch (type) {
    
    case (APRS_WX3):  

        if (debug_level & 1)
           printf("APRS WX3 peet bros U2k (data logging mode) %s:<%s>\n",fill->call_sign,data+2);

        fill->wx_type=WX_TYPE;
        strcpy(fill->wx_station,"U2k");

        /* get last gust speed */
        if (fill->wx_gust_on && !from) {
           /* get last speed */
           last_speed=atof(fill->wx_gust);
           last_speed_time=fill->wx_speed_sec_time;
	}

        /* wind speed */
        if (data[2]!='-') {
           strncpy(temp_data1,data+2,4);
           temp_data1[4]=0;
           sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
           fill->wx_speed_on=1;
           if (from) {
              fill->wx_speed_sec_time = sec_now();
	   } else {
             /* local station */

             computed_gust = compute_gust(atof(fill->wx_speed),last_speed,&last_speed_time);
             fill->wx_speed_sec_time = sec_now();
             fill->wx_gust_on=1;
             sprintf(fill->wx_gust,"%03d",(int)computed_gust);
	   }
	} else {
          if (!from) 
             fill->wx_speed_on=0;
	}
                 
        /* wind direction */
        if (data[8]!='-') {
           strncpy(temp_data1,data+8,2);
           temp_data1[2]=0;
           sprintf(fill->wx_course,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/256.0)*360.0));
           fill->wx_course_on=1; 
	} else {
          sprintf(fill->wx_course,"000");
          if (!from) 
             fill->wx_course_on=0;
	}

        /* outdoor temp */
        if (data[10]!='-') {
           strncpy(temp_data1,data+10,4);
           temp_data1[4]=0;
           sprintf(fill->wx_temp,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
           fill->wx_temp_on=1;
	} else {
          if (!from) 
             fill->wx_temp_on=0;
	}

        /* rain total long term */
        if (data[14]!='-') {
           strncpy(temp_data1,data+14,4);
           temp_data1[4]=0;
           sprintf(fill->wx_rain_total,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
           fill->wx_rain_total_on=1;
           if (!from) {
             /* local station */
             compute_rain(atof(fill->wx_rain_total));
             /*last hour rain */
             sprintf(fill->wx_rain,"%0.2f",rain[hour()]);
             fill->wx_rain_on=1;    
             /*last 24 hour rain */
             sprintf(fill->wx_prec_24,"%0.2f",rain_24);
             fill->wx_prec_24_on=1;  
             /* rain since midnight */
             sprintf(fill->wx_prec_00,"%0.2f",rain_00);
             fill->wx_prec_00_on=1;      
	   }
	} else {
          if (!from) 
             fill->wx_rain_total_on=0;
	}

        /* baro */
        if (data[18]!='-') {
           strncpy(temp_data1,data+18,4);
           temp_data1[4]=0;
           sprintf(fill->wx_baro,"%0.1f",(float)strtol(temp_data1,&temp_conv,16)/10.0);
           fill->wx_baro_on=1;
	}

        /* outdoor humity */
        if (data[26]!='-') {
           strncpy(temp_data1,data+26,4);
           temp_data1[4]=0;
           sprintf(fill->wx_hum,"%03d",(int)((float)strtol(temp_data1,&temp_conv,16)/10.0));
           fill->wx_hum_on=1;
	} else {
          if (!from) 
             fill->wx_hum_on=0;
	}

        /* todays rain total */
        if (data[42]!='-') {
           if (from) {
              strncpy(temp_data1,data+42,4);
              temp_data1[4]=0;
              sprintf(fill->wx_prec_00,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
              fill->wx_prec_00_on=1;
	   }
	} else {
          if (!from) 
             fill->wx_prec_00_on=0;
	}

        break;


    case(APRS_WX4):

        if (debug_level & 1)
           printf("APRS WX4 peet bros UII %s:<%s>\n",fill->call_sign,data); 

        fill->wx_type=WX_TYPE;
        strcpy(fill->wx_station,"UII");
   
        /* wind direction */
        strncpy(temp_data1,data+1,1);
        temp_data1[1]=0;
        sprintf(fill->wx_course,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/16.0)*360.0));
        fill->wx_course_on=1;

        
        /* get last gust speed */
        if (fill->wx_gust_on && !from) {
           /* get last speed */
           last_speed=atof(fill->wx_gust);
           last_speed_time=fill->wx_speed_sec_time;
	}

        /* wind speed */
        strncpy(temp_data1,data+2,2);
        temp_data1[2]=0;
        if (data[0]=='#') {
           /* mph */
           sprintf(fill->wx_speed,"%03d",(int)(float)strtol(temp_data1,&temp_conv,16)); 
	} else {
          /* from kph to mph */
          sprintf(fill->wx_speed,"%03d",(int)(float)(strtol(temp_data1,&temp_conv,16)*0.62137)); 
	}

        fill->wx_speed_on=1;

        if (from) {           
           fill->wx_speed_sec_time = sec_now();
	} else {
          /* local station */

          computed_gust = compute_gust(atof(fill->wx_speed),last_speed,&last_speed_time);
          fill->wx_speed_sec_time = sec_now();
          fill->wx_gust_on=1;
          sprintf(fill->wx_gust,"%03d",(int)computed_gust);
	}
	
        /* outdoor temp */
        if (data[4]!='-') {
           strncpy(temp_data1,data+4,2);
           temp_data1[2]=0;
           sprintf(fill->wx_temp,"%03d",(int)((float)strtol(temp_data1,&temp_conv,16)-56));
           fill->wx_temp_on=1;
	} else {
          if (!from) 
             fill->wx_temp_on=0;
	}

        /* rain div by 100 for readings 0.1 inch */
        if (data[6]!='-') {
           strncpy(temp_data1,data+6,4);
           temp_data1[4]=0;
           sprintf(fill->wx_rain_total,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
           fill->wx_rain_total_on=1;
           if (!from) {
             /* local station */
             compute_rain(atof(fill->wx_rain_total));
             /*last hour rain */
             sprintf(fill->wx_rain,"%0.2f",(float)rain[hour()]);
             fill->wx_rain_on=1;    
             /*last 24 hour rain */
             sprintf(fill->wx_prec_24,"%0.2f",rain_24);
             fill->wx_prec_24_on=1;    
             /* rain since midnight */
             sprintf(fill->wx_prec_00,"%0.2f",rain_00);
             fill->wx_prec_00_on=1;    
	   }
          
	} else {
          if (!from) 
             fill->wx_rain_total_on=0;
	}
        
        fill->wx_baro_on=0;
        fill->wx_hum_on=0;

       break;

    case(APRS_WX5):

        if (debug_level & 1)
           printf("APRS WX5 peet bros U2k Packet (Packet mode) %s:<%s>\n",fill->call_sign,data); 

        fill->wx_type=WX_TYPE;
        strcpy(fill->wx_station,"U2k");

        /* get last gust speed */
        if (fill->wx_gust_on && !from) {
           /* get last speed */
           last_speed=atof(fill->wx_gust);
           last_speed_time=fill->wx_speed_sec_time;
	}

        /* wind speed peak over last 5 min */
        if (data[5]!='-') {
           strncpy(temp_data1,data+5,4);
           temp_data1[4]=0;
           if (from) {
              sprintf(fill->wx_gust,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
              fill->wx_gust_on=1;
              /* this may be the only wind data */  
              sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
              fill->wx_speed_on=1;
	    } else {
              /* local station and may be the only wind data */
              if (len<56) {
                 sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
                 fill->wx_speed_on=1;

                 computed_gust = compute_gust(atof(fill->wx_speed),last_speed,&last_speed_time);
                 fill->wx_speed_sec_time = sec_now();
                 fill->wx_gust_on=1;
                 sprintf(fill->wx_gust,"%03d",(int)computed_gust);
	      }
	    }
	} else {
          if (!from) 
             fill->wx_gust_on=0;
	}
        
        /* wind direction */
        if (data[11]!='-') {
           strncpy(temp_data1,data+11,2);
           temp_data1[2]=0;
           sprintf(fill->wx_course,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/256.0)*360.0));
           fill->wx_course_on=1;
	} else {
          sprintf(fill->wx_course,"000");
          if (!from) 
             fill->wx_course_on=0;
	}

        /* outdoor temp */
        if (data[13]!='-') {
           strncpy(temp_data1,data+13,4);
           temp_data1[4]=0;
           sprintf(fill->wx_temp,"%03d",(int)((float)((strtol(temp_data1,&temp_conv,16)<<16)/65636)/10.0));
           fill->wx_temp_on=1;
	} else {
          if (!from) 
             fill->wx_temp_on=0;     
	}

        /* todays rain total (on some units) */
        if (data[49]!='-') {
           if (from) {
              strncpy(temp_data1,data+49,4);
              temp_data1[4]=0;
              sprintf(fill->wx_prec_00,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
              fill->wx_prec_00_on=1;
	   }
	} else {
          if (!from) 
             fill->wx_prec_00_on=0;
	}

        /* rain total long term */
        if (data[17]!='-') {
           strncpy(temp_data1,data+17,4);
           temp_data1[4]=0;
           sprintf(fill->wx_rain_total,"%0.2f",(float)strtol(temp_data1,&temp_conv,16)/100.0);
           fill->wx_rain_total_on=1;
           if (!from) {
             /* local station */
             compute_rain(atof(fill->wx_rain_total));
             /*last hour rain */
             sprintf(fill->wx_rain,"%0.2f",rain[hour()]);
             fill->wx_rain_on=1;    
             /*last 24 hour rain */
             sprintf(fill->wx_prec_24,"%0.2f",rain_24);
             fill->wx_prec_24_on=1;    
             /* rain since midnight */
             sprintf(fill->wx_prec_00,"%0.2f",rain_00);
             fill->wx_prec_00_on=1;     
	   }
	} else {
          if (!from) 
             fill->wx_rain_total_on=0;
	}

        /* baro */
        if (data[21]!='-') {
           strncpy(temp_data1,data+21,4);
           temp_data1[4]=0;
           sprintf(fill->wx_baro,"%0.1f",(float)strtol(temp_data1,&temp_conv,16)/10.0);
           fill->wx_baro_on=1;
	} else {
          if (!from) 
             fill->wx_baro_on=0;
	}

        /* outdoor humity */
        if (data[37]!='-') {
           strncpy(temp_data1,data+37,4);
           temp_data1[4]=0;
           sprintf(fill->wx_hum,"%03d",(int)((float)strtol(temp_data1,&temp_conv,16)/10.0));
           fill->wx_hum_on=1;
	} else {
          if (!from) 
             fill->wx_hum_on=0;
	}

        /* 1 min wind speed avg */
        if (len>53 && data[53]!='-') {
           strncpy(temp_data1,data+53,4);
           temp_data1[4]=0;
           sprintf(fill->wx_speed,"%03d",(int)(((float)strtol(temp_data1,&temp_conv,16)/10.0)*0.62137));
           fill->wx_speed_on=1;
           if (from) {
              fill->wx_speed_sec_time = sec_now();
	   } else {
             /* local station */
                
             computed_gust = compute_gust(atof(fill->wx_speed),last_speed,&last_speed_time);
             fill->wx_speed_sec_time = sec_now();
             fill->wx_gust_on=1;
             sprintf(fill->wx_gust,"%03d",(int)computed_gust);
	   }
	} else {
          if (!from) {
             if (len>53) 
                fill->wx_speed_on=0;
	  }
	}

        break;

    case(APRS_WX6):

        if (debug_level & 1)
           printf("Qualimetrics Q-Net %s:<%s>\n",fill->call_sign,data); 

        fill->wx_type=WX_TYPE;
        strcpy(fill->wx_station,"Q-N");

        sscanf(data,"%s %d %s %d %s %d",temp,&temp1,temp,&temp2,temp,&temp3); 

        /* outdoor temp */
        sprintf(fill->wx_temp,"%03d",(int)(temp2/10.0));
        fill->wx_temp_on=1;

        /* baro */
        sprintf(fill->wx_baro,"%0.1f",((float)temp3/100.0)*33.864);
        fill->wx_baro_on=1;


        /* outdoor humity */
        sprintf(fill->wx_hum,"%03d",temp1);
        fill->wx_hum_on=1;

        if (!from) {
           fill->wx_gust_on=0;
           fill->wx_course_on=0;
           fill->wx_rain_on=0;
           fill->wx_prec_00_on=0;
           fill->wx_prec_24_on=0;
           fill->wx_rain_total_on=0;
           fill->wx_gust_on=0;
           fill->wx_speed_on=0;
	}
        break;

       default:break;
 }
}


/***********************************************************/
/* Decode WX data line                                     */
/* wx_line: raw wx data to decode                          */
/***********************************************************/
void wx_decode(char *wx_line, int port)
{
 DataRow fill;
 int record;
 int decoded;
 int find;
 int i;
 int len;
 char time_data[MAX_TIME];
 
 find=0;
 len=strlen(wx_line);
 if (len>10) {
    strcpy(fill.call_sign,my_callsign);

    record = mem_find_data(fill);
    if (record != -1l) {
       /* current data */
       mem_get_data(&fill,record);
     

       decoded=0;
       /* Ok decode wx data */
       if (wx_line[0]=='!' && wx_line[1]=='!' && is_xnum_or_dash(wx_line+2,40)) {
          /* Found Peet Bros U2k */
          /*printf("Found Peet Bros U2k WX:%s\n",wx_line+2);*/
          strcpy(wx_station_type,langcode("WXPUPSI011"));
          strcpy(raw_wx_string,wx_line);
          strcpy(fill.wx_time,get_time(time_data));
          fill.wx_sec_time=sec_now();
          fill.wx_data=1;
          wx_fill_data(0,APRS_WX3,wx_line,&fill);
          decoded=1; 
          } else { 
            if (((wx_line[0]=='#') || (wx_line[0]=='*')) && is_xnum_or_dash(wx_line+1,13)) {
               /* Found Peet Bros raw U2 data */
               strcpy(wx_station_type,langcode("WXPUPSI012"));
               /*printf("Found Peet Bros raw U2 data WX#:%s\n",wx_line+1);*/
               strcpy(raw_wx_string,wx_line);
               strcpy(fill.wx_time,get_time(time_data));
               fill.wx_sec_time=sec_now();
               fill.wx_data=1;
               wx_fill_data(0,APRS_WX4,wx_line,&fill);
               decoded=1; 
	    } else { 
              if (strncmp("$ULTW",wx_line,5)==0 && is_xnum_or_dash(wx_line+5,44)) {
                 /* Found Peet Bros raw U2 data */
                 strcpy(wx_station_type,langcode("WXPUPSI013"));
                 /*printf("Found Peet Bros Ultmeter Packet data WX#:%s\n",wx_line+5);*/
                 strcpy(raw_wx_string,wx_line);
                 fill.wx_sec_time=sec_now();
                 fill.wx_data=1;
                 wx_fill_data(0,APRS_WX5,wx_line,&fill);
                 decoded=1; 

	      } else {
                if (wx_line[2]==' ' && wx_line[5]==' ' && wx_line[8]=='/' && wx_line[11]=='/' 
                    && wx_line[14]==' ' && wx_line[17]==':' && wx_line[20]==':' 
                    && strncmp(" #0:",wx_line+23,4)==0
                    ) { 
                    find=0;
                    for (i=len;i>23 && !find;i--) {
                      if (wx_line[i]==0x03) {
                         find=1;
                         wx_line[i]=0l;
		      }
		    }
                    if (find) {   
                       /* found Qualimetrics Q-Net station */
                       strcpy(wx_station_type,langcode("WXPUPSI016"));
                       /*printf("Found Qualimetrics Q-Net station data WX#:%s\n",wx_line+23);*/
                       strcpy(fill.wx_time,get_time(time_data));
                       fill.wx_sec_time=sec_now();
                       fill.wx_data=1;
                       wx_fill_data(0,APRS_WX6,wx_line+24,&fill);
                       decoded=1; 
		    }

		} else {
                  printf("Unknown WX DATA:%s\n",wx_line);
                  memset(raw_wx_string,'\0',sizeof(raw_wx_string));
		}
	      }
	    }
	  }

      if (decoded) {
         /* save data back */
      
         XmTextFieldSetString(text,langcode("BBARSTA032"));
         XmUpdateDisplay(text);  
     
         mem_replace_data(fill,record);

         /* redraw now */
         redraw_on_new_data=2;
         fill_wx_data();
      }
    }
 }
}


/***********************************************************/
/* fill string with WX data for transmit                   */
/*                            */
/***********************************************************/

time_t wx_tx_data1(char *st)
{
 DataRow fill;
 int record;
 time_t wx_time;
 char temp[100];

 strcpy(st,"");
 wx_time=0;
 strcpy(fill.call_sign,my_callsign);
 
 record = mem_find_data(fill);
 if (record != -1l) {
    /* current data */
    mem_get_data(&fill,record);
    if (fill.wx_data) {
       /* station has wx data */
       if (fill.wx_course_on && fill.wx_speed_on) {
          /* we have enough wx_data */
          wx_time=fill.wx_sec_time;
          sprintf(st,"%s/%s",fill.wx_course,fill.wx_speed);
       } else {
         /* we don't have enough wx_data, must be from a Qualimetrics Q-Net */
          wx_time=fill.wx_sec_time;
          sprintf(st,".../...");
       }

       if (fill.wx_gust_on) {
          sprintf(temp,"g%s",fill.wx_gust);
          if (strlen(temp)>4) 
             strcpy(temp,"g999");
          strcat(st,temp);
	  } else strcat(st,"g...");
          
          
       if (fill.wx_temp_on) {
          sprintf(temp,"t%s",fill.wx_temp);
          strcat(st,temp);
       } else strcat(st,"t...");


       if (fill.wx_rain_on) {
          sprintf(temp,"r%03d",(int)(atof(fill.wx_rain)*100.0));
          if (strlen(temp)>4) 
             strcpy(temp,"r999");
          strcat(st,temp);
       } else strcat(st,"r...");

       if (fill.wx_prec_00_on) {
          sprintf(temp,"p%03d",(int)(atof(fill.wx_prec_00)*100.0));
          if (strlen(temp)>4)
             strcpy(temp,"p999");
          strcat(st,temp);  
       } else strcat(st,"p...");

       if (fill.wx_prec_24_on) {
          sprintf(temp,"P%03d",(int)(atof(fill.wx_prec_24)*100.0));
          if (strlen(temp)>4)
             strcpy(temp,"P999");
          strcat(st,temp);
       } else strcat(st,"P...");

 
       if (fill.wx_hum_on) {
           if (atoi(fill.wx_hum)>99) {
              sprintf(temp,"h00");
	   } else sprintf(temp,"h%02d",atoi(fill.wx_hum));
           strcat(st,temp); 
       } else strcat(st,"h..");
	 
         
       if (fill.wx_baro_on) {
          sprintf(temp,"b%05d",(int)(atof(fill.wx_baro)*10.0));
          if (strlen(temp)>6)
             strcpy(temp,"b99999");
          strcat(st,temp);
       } else strcat(st,"b.....");
         
       sprintf(temp,"%c%s",fill.wx_type,fill.wx_station);
       strcat(st,temp);
    }
 }

return(wx_time);
}


/***********************************************************/
/* transmit raw wx data                                    */
/*                                                         */
/***********************************************************/
void tx_raw_wx_data(void)
{

 printf("Send Raw WX\n");
 if (strlen(raw_wx_string)>10) {
    output_my_data(raw_wx_string,-1,0);
    printf("Raw out <%s>\n",raw_wx_string);
 }

}
