/*
XASTIR, Amateur Station Tracking and Information Reporting
Copyright (C) 1999  Frank Giannandrea, KC0DGE

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

More information on the program can be found at:
Web,
 http://www.eazy.net/users/fgiannan/xasitr/
Email,
 fgiannan@eazy.net
*/

#include <string.h>
#include <stdlib.h>
#include <termios.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>
#include <Xm/Frame.h>


#include "main.h"
#include "xastir.h"
#include "alert.h"

void fill_wx_data(void);

extern char *get_time(char *time_here);
extern time_t sec_now(void);
extern int redraw_on_new_data;
extern Widget GetTopShell(Widget w);
extern void pos_dialog(Widget w);
extern int units_english_metric;
extern char wx_station_type[];

/* from lang.c */
extern char *langcode(char *code);
extern char langcode_hotkey(char *code);

/************ Weather Alerts ****************/
Widget wx_alert_shell = (Widget)NULL;
static Widget wx_alert_list;

void wx_alert_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{
 Widget shell = (Widget) clientData;
 XtPopdown(shell);
 XtDestroyWidget(shell);
 wx_alert_shell = (Widget)NULL;
}

void wx_alert_update_list(void)
{
    int n, item_count;
    char temp[600];
    XmString item;

    if (wx_alert_shell) {
	XtVaGetValues(wx_alert_list, XmNitemCount, &item_count, NULL);
	for (n = 0; n < alert_list_count; n++) {
		    sprintf(temp, "%c,%-9s>%-9s:%-9s,%-20s,%s",
				    alert_list[n].flags[0], alert_list[n].from, alert_list[n].to,
				    alert_list[n].activity, alert_list[n].alert_tag, alert_list[n].title);
		    item = XmStringCreateLtoR(temp, XmFONTLIST_DEFAULT_TAG);
		    if (item_count <= n)
		      XmListAddItemUnselected(wx_alert_list, item, 0);
		    else
		      XmListReplaceItemsPosUnselected(wx_alert_list, &item, 1, n+1);
	}
	if (alert_list_count < item_count)
	    XmListDeleteItemsPos(wx_alert_list, item_count - alert_list_count, alert_list_count+1);
    }
}

void Display_Wx_Alert(Widget wdgt, XtPointer clientData, XtPointer callData)
{
    static Widget pane, form, mess, button_cancel;
    Atom delw;

    if(!wx_alert_shell) {
	wx_alert_shell = XtVaCreatePopupShell(langcode("WPUPWXA001"), xmDialogShellWidgetClass, GetTopShell(wdgt),
					       XmNdeleteResponse, XmDESTROY,
					       XmNdefaultPosition, False,
					       NULL);
	pane = XtVaCreateWidget("pane", xmPanedWindowWidgetClass, wx_alert_shell,
							XmNsashWidth, 1,
							XmNsashHeight, 1,
							NULL);

	form =  XtVaCreateWidget("form2", xmFormWidgetClass, pane,
							 XmNfractionBase, 5,
							 NULL);

	wx_alert_list = XmCreateScrolledList(form, "wx_alert_list", NULL, 0);
	XtVaSetValues(wx_alert_list,
			      XmNvisibleItemCount, 13,
			      XmNtraversalOn, False,
			      XmNshadowThickness, 3,
                              XmNbackground, colors[0x0f],
			      XmNselectionPolicy, XmMULTIPLE_SELECT,
			      XmNscrollBarPlacement, XmBOTTOM_LEFT,
			      XmNlistSizePolicy, XmVARIABLE,
			      XmNscrollBarDisplayPolicy, XmSTATIC,
			      NULL);

	wx_alert_update_list();

	mess = XtVaCreateManagedWidget(langcode("WPUPWXA002"), xmLabelWidgetClass, form,
							       XmNtopAttachment, XmATTACH_FORM,
							       XmNtopOffset, 5,
							       XmNbottomAttachment, XmATTACH_NONE,
							       XmNleftAttachment, XmATTACH_FORM,
							       XmNrightAttachment, XmATTACH_FORM,
							       NULL);

	XtVaSetValues(XtParent(wx_alert_list),
			      XmNtopAttachment, XmATTACH_WIDGET,
			      XmNtopWidget, mess,
			      XmNtopOffset, 5,
			      XmNbottomAttachment, XmATTACH_NONE,
			      XmNrightAttachment, XmATTACH_FORM,
			      XmNleftAttachment, XmATTACH_FORM,
			      NULL);

	button_cancel = XtVaCreateManagedWidget(langcode("UNIOP00003"),xmPushButtonGadgetClass, form,
					        XmNtopAttachment, XmATTACH_FORM,
						XmNtopOffset, 270,
						XmNbottomAttachment, XmATTACH_FORM,
						XmNleftAttachment, XmATTACH_POSITION,
						XmNleftPosition, 2,
						XmNrightAttachment, XmATTACH_POSITION,
						XmNrightPosition, 3,
						XmNshowAsDefault, True,
						XmNdefaultButtonShadowThickness, 1,
						NULL);

	XtAddCallback(button_cancel, XmNactivateCallback, wx_alert_destroy_shell, wx_alert_shell);
	pos_dialog(wx_alert_shell);

	delw = XmInternAtom(XtDisplay(wx_alert_shell), "WM_DELETE_WINDOW", False);
	XmAddWMProtocolCallback(wx_alert_shell, delw, wx_alert_destroy_shell, (XtPointer)wx_alert_shell);

	XtManageChild(form);
	XtManageChild(wx_alert_list);
	XtManageChild(pane);

	XtPopup(wx_alert_shell, XtGrabNone);
        fix_dialog_vsize(wx_alert_shell);         
    }
} /* Display_Wx_Alert */



/**** WX Station *******/
Widget wx_station_dialog=(Widget)NULL;
Widget WX_type_data;
Widget WX_temp_data;
Widget WX_wind_cse_data;
Widget WX_wind_spd_data;
Widget WX_wind_gst_data;
Widget WX_rain_data;
Widget WX_to_rain_data;
Widget WX_rain_h_data;
Widget WX_rain_24_data;
Widget WX_baro_data;
Widget WX_humidity_data;
Widget WX_speed_label;
Widget WX_gust_label;
Widget WX_temp_label;
Widget WX_rain_label;
Widget WX_to_rain_label;
Widget WX_rain_h_label;
Widget WX_rain_24_label;

void WX_station_destroy_shell(Widget widget, XtPointer clientData, XtPointer callData)
{ 
 Widget shell = (Widget) clientData;
 XtPopdown(shell);
 XtDestroyWidget(shell);
 wx_station_dialog = (Widget)NULL;
}


void WX_station_change_data(Widget widget, XtPointer clientData, XtPointer callData)
{

 WX_station_destroy_shell(widget,clientData,callData);
}

void WX_station(Widget w, XtPointer clientData, XtPointer callData)
{static Widget  pane, form, form1, button_close, frame, 
                WX_type, temp, wind_cse, wind_deg, wind_spd, wind_gst, 
                rain, to_rain, rain_h, rain_24, baro, baro_n, 
                humidity, humidity_n, sts;

 Atom delw;


 if(!wx_station_dialog)   
   {wx_station_dialog = XtVaCreatePopupShell(langcode("WXPUPSI000"),xmDialogShellWidgetClass,GetTopShell(w),
                                  XmNdeleteResponse,XmDESTROY,
                                  XmNdefaultPosition, False,
                                  NULL);

    pane = XtVaCreateWidget("pane",xmPanedWindowWidgetClass, wx_station_dialog,
                          XmNsashWidth, 1,
                          XmNsashHeight, 1,
                         NULL);



    form =  XtVaCreateWidget("form2",xmFormWidgetClass,pane,
                            XmNfractionBase, 5, 
                            NULL);

    WX_type = XtVaCreateManagedWidget(langcode("WXPUPSI001"),xmLabelWidgetClass, form,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 10,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_type_data = XtVaCreateManagedWidget("type data", xmTextFieldWidgetClass, form,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 50,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment,XmATTACH_WIDGET,
                                      XmNleftWidget, WX_type,
                                      XmNleftOffset, 5,
                                      XmNtopAttachment,XmATTACH_FORM,                    
                                      XmNbottomAttachment,XmATTACH_NONE,                                
                                      XmNrightAttachment,XmATTACH_NONE,
                                      NULL);

    frame = XtVaCreateManagedWidget("frame1", xmFrameWidgetClass, form,
                                    XmNtopAttachment,XmATTACH_WIDGET,
                                    XmNtopWidget, WX_type_data,
                                    XmNtopOffset,10,
                                    XmNbottomAttachment,XmATTACH_NONE,
                                    XmNleftAttachment, XmATTACH_FORM,
                                    XmNleftOffset, 10,
                                    XmNrightAttachment,XmATTACH_FORM,
                                    XmNrightOffset, 10,
                                    NULL);

    sts  = XtVaCreateManagedWidget(langcode("WXPUPSI002"),xmLabelWidgetClass,frame,
                                    XmNchildType, XmFRAME_TITLE_CHILD,
                                    NULL);

    form1 =  XtVaCreateWidget("form1",xmFormWidgetClass, frame,
                            XmNfractionBase, 5, 
                            NULL);


    wind_cse = XtVaCreateManagedWidget(langcode("WXPUPSI003"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_wind_cse_data = XtVaCreateManagedWidget("wc data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 3,
                                      XmNmaxLength, 3,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_FORM,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);

    wind_deg = XtVaCreateManagedWidget(langcode("UNIOP00024"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_FORM,
                                      XmNtopOffset, 12,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_wind_cse_data,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    wind_spd = XtVaCreateManagedWidget(langcode("WXPUPSI004"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, wind_cse,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_wind_spd_data = XtVaCreateManagedWidget("ws data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 3,
                                      XmNmaxLength, 3,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, wind_cse,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);

     WX_speed_label= XtVaCreateManagedWidget("speed label",xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
                                      XmNshadowThickness,      0,
                                      XmNcolumns,5,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, wind_cse, 
                                      XmNtopOffset, 8,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_wind_spd_data,
                                      XmNleftOffset, 0,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    wind_gst = XtVaCreateManagedWidget(langcode("WXPUPSI005"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, wind_spd,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_wind_gst_data = XtVaCreateManagedWidget("wg data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 3,
                                      XmNmaxLength, 3,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, wind_spd,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);

     WX_gust_label= XtVaCreateManagedWidget("gust label",xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
                                      XmNshadowThickness,      0,
                                      XmNcolumns,5,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, wind_spd, 
                                      XmNtopOffset, 8,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_wind_gst_data,
                                      XmNleftOffset, 0,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    temp = XtVaCreateManagedWidget(langcode("WXPUPSI006"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, wind_gst,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_temp_data = XtVaCreateManagedWidget("temp data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 8,
                                      XmNmaxLength, 8,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, wind_gst,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);

     WX_temp_label= XtVaCreateManagedWidget("temp label",xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
                                      XmNshadowThickness,      0,
                                      XmNcolumns,5,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, wind_gst, 
                                      XmNtopOffset, 8,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_temp_data,
                                      XmNleftOffset, 0,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    rain = XtVaCreateManagedWidget(langcode("WXPUPSI007"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, temp,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_rain_data = XtVaCreateManagedWidget("rain data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 8,
                                      XmNmaxLength, 10,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, temp,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);

     WX_rain_label= XtVaCreateManagedWidget("rain label",xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
                                      XmNshadowThickness,      0,
                                      XmNcolumns,5,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, temp, 
                                      XmNtopOffset, 8,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_rain_data,
                                      XmNleftOffset, 0,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    to_rain = XtVaCreateManagedWidget(langcode("WXPUPSI008"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, rain,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_to_rain_data = XtVaCreateManagedWidget("today rain data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 8,
                                      XmNmaxLength, 10,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, rain,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);


     WX_to_rain_label= XtVaCreateManagedWidget("to label",xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
                                      XmNshadowThickness,      0,
                                      XmNcolumns,20,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, rain, 
                                      XmNtopOffset, 8,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_to_rain_data,
                                      XmNleftOffset, 0,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    rain_h = XtVaCreateManagedWidget(langcode("WXPUPSI014"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, to_rain,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_rain_h_data = XtVaCreateManagedWidget("hour rain data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 8,
                                      XmNmaxLength, 10,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, to_rain,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);


     WX_rain_h_label= XtVaCreateManagedWidget("hour label",xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
                                      XmNshadowThickness,      0,
                                      XmNcolumns,20,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, to_rain, 
                                      XmNtopOffset, 8,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_rain_h_data,
                                      XmNleftOffset, 0,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    rain_24 = XtVaCreateManagedWidget(langcode("WXPUPSI015"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, rain_h,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_rain_24_data = XtVaCreateManagedWidget("24h rain data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 8,
                                      XmNmaxLength, 10,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, rain_h,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);


     WX_rain_24_label= XtVaCreateManagedWidget("24h label",xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, STIPPLE,
                                      XmNshadowThickness,      0,
                                      XmNcolumns,20,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, rain_h, 
                                      XmNtopOffset, 8,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_rain_24_data,
                                      XmNleftOffset, 0,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    baro = XtVaCreateManagedWidget(langcode("WXPUPSI009"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, rain_24,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_baro_data = XtVaCreateManagedWidget("Baro data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 8,
                                      XmNmaxLength, 8,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, rain_24,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);

    baro_n = XtVaCreateManagedWidget(langcode("UNIOP00025"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, rain_24,
                                      XmNtopOffset, 12,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_baro_data,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);

    humidity = XtVaCreateManagedWidget(langcode("WXPUPSI010"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, baro,
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_FORM,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    WX_humidity_data = XtVaCreateManagedWidget("Humidity data", xmTextFieldWidgetClass, form1,
                                      XmNeditable,   False,
                                      XmNcursorPositionVisible, False,
                                      XmNsensitive, True,
				      XmNshadowThickness,	1,
                                      XmNcolumns, 8,
                                      XmNmaxLength, 8,
                                      XmNtopOffset, 6, 
                                      XmNbackground, colors[0x0f],
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      XmNtopAttachment,XmATTACH_WIDGET,
                                      XmNtopWidget, baro,                    
                                      XmNbottomAttachment,XmATTACH_NONE,
                                      NULL);

    humidity_n = XtVaCreateManagedWidget(langcode("UNIOP00026"),xmLabelWidgetClass, form1,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, baro, 
                                      XmNtopOffset, 12,
                                      XmNbottomAttachment, XmATTACH_NONE,
                                      XmNleftAttachment, XmATTACH_WIDGET,
                                      XmNleftWidget, WX_humidity_data,
                                      XmNleftOffset, 5,
                                      XmNrightAttachment, XmATTACH_NONE,
                                      NULL);


    button_close = XtVaCreateManagedWidget(langcode("UNIOP00003"),xmPushButtonGadgetClass, form,
                                      XmNtopAttachment, XmATTACH_WIDGET,
                                      XmNtopWidget, frame, 
                                      XmNtopOffset, 10,
                                      XmNbottomAttachment, XmATTACH_FORM,
                                      XmNleftAttachment, XmATTACH_POSITION,
                                      XmNleftPosition, 2,
                                      XmNrightAttachment, XmATTACH_POSITION,
                                      XmNrightPosition, 3,
                                      XmNshowAsDefault, False,
                                      XmNdefaultButtonShadowThickness, 1,
                                      NULL);


   XtAddCallback(button_close, XmNactivateCallback, WX_station_destroy_shell, wx_station_dialog);

   delw = XmInternAtom(XtDisplay(wx_station_dialog),"WM_DELETE_WINDOW", False);
   XmAddWMProtocolCallback(wx_station_dialog, delw, WX_station_destroy_shell, (XtPointer)wx_station_dialog);

   XtManageChild(form);  
   XtManageChild(form1);  
   XtManageChild(pane);

   fill_wx_data();

   XtPopup(wx_station_dialog,XtGrabNone);
   fix_dialog_size(wx_station_dialog);
   }
}


void fill_wx_data(void)
{DataRow fill;
 int record;
 char temp[20];

 if (wx_station_dialog!=NULL) {

     strcpy(fill.call_sign,my_callsign);

     record = mem_find_data(fill);
     if (record != -1l) {
        /* current data */
        mem_get_data(&fill,record);

        if (strlen(wx_station_type)>1) {
           XmTextFieldSetString(WX_type_data,wx_station_type);
	} else XmTextFieldSetString(WX_type_data,"");
        XtManageChild(WX_type_data);

        if (fill.wx_data) {
  
            if (fill.wx_temp_on) {
               if (!units_english_metric) {
                   sprintf(temp,"%03d",(int)(((atof(fill.wx_temp)-32)*5.0)/9.0));
                   XmTextFieldSetString(WX_temp_data,temp);
	       } else XmTextFieldSetString(WX_temp_data,fill.wx_temp);  
                
	    } else XmTextFieldSetString(WX_temp_data,"");

            XtManageChild(WX_temp_data);

            if (fill.wx_course_on) {
                XmTextFieldSetString(WX_wind_cse_data,fill.wx_course);
	    } else XmTextFieldSetString(WX_wind_cse_data,"");
            XtManageChild(WX_wind_cse_data);

            if (fill.wx_speed_on) {
               if (!units_english_metric) {
                  sprintf(temp,"%03d",(int)(atof(fill.wx_speed)*1.6094));
                  XmTextFieldSetString(WX_wind_spd_data,temp);
	       } else XmTextFieldSetString(WX_wind_spd_data,fill.wx_speed);
	    } else XmTextFieldSetString(WX_wind_spd_data,"");
            
            XtManageChild(WX_wind_spd_data);

            if (fill.wx_gust_on) {
               if (!units_english_metric) {
                  sprintf(temp,"%03d",(int)(atof(fill.wx_gust)*1.6094));
                  XmTextFieldSetString(WX_wind_spd_data,temp);
	       } else XmTextFieldSetString(WX_wind_gst_data,fill.wx_gust);
	    } else XmTextFieldSetString(WX_wind_gst_data,"");
            XtManageChild(WX_wind_gst_data);

            if (fill.wx_rain_total_on) {
               if (!units_english_metric) {
                  sprintf(temp,"%0.2f",atof(fill.wx_rain_total)*.254);
                  XmTextFieldSetString(WX_rain_data,temp);
	       } else XmTextFieldSetString(WX_rain_data,fill.wx_rain_total);
	    } else XmTextFieldSetString(WX_rain_data,"");
            XtManageChild(WX_rain_data);

            if (fill.wx_rain_on) {
               if (!units_english_metric) {
                  sprintf(temp,"%0.2f",atof(fill.wx_rain)*.254);
                  XmTextFieldSetString(WX_rain_h_data,temp);
	       } else XmTextFieldSetString(WX_rain_h_data,fill.wx_rain);
	    } else XmTextFieldSetString(WX_rain_h_data,"");
            XtManageChild(WX_rain_h_data);

            if (fill.wx_prec_24_on) {
               if (!units_english_metric) {
                  sprintf(temp,"%0.2f",atof(fill.wx_prec_24)*.254);
                  XmTextFieldSetString(WX_rain_24_data,temp);
	       } else XmTextFieldSetString(WX_rain_24_data,fill.wx_prec_24);
	    } else XmTextFieldSetString(WX_rain_24_data,"");
            XtManageChild(WX_rain_24_data);

            if (fill.wx_prec_00_on) {
               if (!units_english_metric) {
                  sprintf(temp,"%0.2f",atof(fill.wx_prec_00)*.254);
                  XmTextFieldSetString(WX_to_rain_data,temp);
	       } else XmTextFieldSetString(WX_to_rain_data,fill.wx_prec_00);
	    } else XmTextFieldSetString(WX_to_rain_data,"");
            XtManageChild(WX_rain_data);

            if (fill.wx_baro_on) {
                XmTextFieldSetString(WX_baro_data,fill.wx_baro);
	    } else XmTextFieldSetString(WX_baro_data,"");
            XtManageChild(WX_baro_data);

            if (fill.wx_hum_on) {
                XmTextFieldSetString(WX_humidity_data,fill.wx_hum);
	    } else XmTextFieldSetString(WX_humidity_data,"");
            XtManageChild(WX_humidity_data);

	}
     }
     
   /* labels */
   if (!units_english_metric) {
        XmTextFieldSetString(WX_speed_label,langcode("UNIOP00012"));
   } else XmTextFieldSetString(WX_speed_label,langcode("UNIOP00013"));
   XtManageChild(WX_speed_label);

   if (!units_english_metric) {
        XmTextFieldSetString(WX_gust_label,langcode("UNIOP00012"));
   } else XmTextFieldSetString(WX_gust_label,langcode("UNIOP00013"));
   XtManageChild(WX_gust_label);


   if (!units_english_metric) {
      XmTextFieldSetString(WX_temp_label,langcode("UNIOP00014"));
   } else XmTextFieldSetString(WX_temp_label,langcode("UNIOP00015"));
   XtManageChild(WX_temp_label);

   if (!units_english_metric) {
      XmTextFieldSetString(WX_rain_label,langcode("UNIOP00016"));
   } else XmTextFieldSetString(WX_rain_label,langcode("UNIOP00017"));
   XtManageChild(WX_rain_label);
             
   if (!units_english_metric) {
      XmTextFieldSetString(WX_to_rain_label,langcode("UNIOP00022"));
   } else XmTextFieldSetString(WX_to_rain_label,langcode("UNIOP00023"));
   XtManageChild(WX_to_rain_label);

   if (!units_english_metric) {
      XmTextFieldSetString(WX_rain_h_label,langcode("UNIOP00020"));
   } else XmTextFieldSetString(WX_rain_h_label,langcode("UNIOP00021"));
   XtManageChild(WX_rain_h_label);

   if (!units_english_metric) {
      XmTextFieldSetString(WX_rain_24_label,langcode("UNIOP00018"));
   } else XmTextFieldSetString(WX_rain_24_label,langcode("UNIOP00019"));
   XtManageChild(WX_rain_24_label);


 }
}
